/*
 * Use SWI instruction.
 * See gdb/sim/arm/armemu.c:ARMul_Emulate32(),armos.c:ARMul_OSHandleSWI()
 * (case 0xf0:)
 */

#define SWI_Exit  0x11
#define SWI_Open  0x66
#define SWI_Close 0x68
#define SWI_Write 0x69
#define SWI_Read  0x6a

#define TOPBITS_AL (14 << 28) /* condition code */
#define SWI(arg) (TOPBITS_AL | (0xf0 << 20) | (arg))

	.section .text

	.globl	_start
	.type	_start, %function
_start:
	ldr	sp, _stack_addr
	bl	main
	mov	r0, #0
	/* fall through */

	.globl	__exit
	.type	__exit, %function
__exit:
	.long	SWI(SWI_Exit)
	mov	pc, lr

	.globl	__read
	.type	__read, %function
__read:
	.long	SWI(SWI_Read)
	mov	pc, lr

	.globl	__write
	.type	__write, %function
__write:
	.long	SWI(SWI_Write)
	mov	pc, lr

	.globl	__open
	.type	__open, %function
__open:
	.long	SWI(SWI_Open)
	mov	pc, lr

	.globl	__close
	.type	__close, %function
__close:
	.long	SWI(SWI_Close)
	mov	pc, lr

	.align 4
_stack_addr:
	.long	_estack
